
% Initial set up.

clear all;
close all;
clc;
global xx yy zz s Val  delta gamma Q w alpha n ns np Dir cparam th
global q_opt_iter ndir Jtrack Qtrack Vtrack rope DG dtrack
global ropt is in
tic


% Maximization parameters.

tol      =  1e-4; % tolerance level
Maxiter  =  150;    % maximium iteration #
iter     =  0;    % iteration counter
neg_eps  = -1e-3; % for checking the constraints
ifirst   =  0;    % Set First run indicator here.
start    =  true;
counter  =  0;
cparam   =  0.000001;

% Number of states/directions/people

ns   = 3;           % number of states.
ndir = 100;          % number of directions.
np   = 2;           % number of people.
nv   = np*(1+ns);   % number of variables.


% Create the initial unit quarter-hyper-circle

cen = [0 0 0]; 
rad = 2.2;                      % center radius.
inc = (90/ndir).*[1:1:ndir];    % increment in the angle.

% psi gives split between 1 and 2. 
% A low psi gives more to 1.

psi = [0 inc];

% psi is in degrees. To express in radians:

psir = degtorad(psi);

% To normalize to 1.

psi1n = psi./inc(ndir);

psia  = 90 -psi;
psi1na=psia./inc(ndir);


Dir = [];  % direction
Cir = [];  % circle


% Set hyperplane directions.


    for j=1:ndir+1
        
        y(j)   = cosd(psi(j)); 

        z(j)   = sind(psi(j));
        
    end


% Record totals.

ic    = ndir+1 ;
Tdir  = ic;


Dir  = [y; z]




% Set parameters that we need.

% Utility.

% This version: alpha = 1.5, theta = 0.5.

delta = 0.8; % discount
alpha = 1.5; % sigma in text. 1/alpha = IES
theta = 8;   % Risk aversion. CRRA    = 1 - theta*(1-alpha);

th    = (theta-1)/theta;


% Note: theta = 1 is expected utility with CRRA = 1/IES.

% Lower values of theta => more risk aversion.

% Higher values of alpha => lower IES (and more risk aversion).


% Transition probability matrix

state = [1:1:ns];           % two states
gamma = 1.2*ones(ns,1);     % endowment
Q     = zeros(ns);          % initialize transition probability matrix

% Define probabilities: row is current and column is next state

stay = 0.9;
go   = (1-stay)/(ns-1);


for i=1:ns
for j=1:ns
    if j==i
        Q(i,j) = stay;
    else
        Q(i,j) = go;
    end
end
end



% Set outside options.
% w(s,p) = state s and person p.



for i=1:np
    for j=1:np
        if j == i
            w(i,j) = ((0.52*gamma(j))^(1-alpha))/(1-alpha);
	        w2(i,j)= ((0.52*gamma(j))^(1-alpha))/(1-alpha);        
        else
            w(i,j) = ((0.35*gamma(j))^(1-alpha))/(1-alpha);
            %w2(i,j)= ((0.355*gamma(j))^(1-alpha))/(1-alpha);
            w2(i,j)= ((0.42*gamma(j))^(1-alpha))/(1-alpha);
        end
	        
    end
end

for j=1:np
    w(ns,j) = ((0.35*gamma(j))^(1-alpha))/(1-alpha);
    %w2(ns,j)= ((0.45*gamma(j))^(1-alpha))/(1-alpha);
    w2(ns,j)= ((0.44*gamma(j))^(1-alpha))/(1-alpha);
end


% Take a look to check.

w
w2


% We now compute Valbarbar and Val.

% For value functions:

% Arguments  :       s  = shock,, s in state.
%                    i  = direction index, ic=1,...,Tdir.



% First fix Valbarbar (Dbarbar in paper) using procedure described in
% paper. Also determine Valup (barD in the paper).

for s = state
        
        for n = 1:Tdir
            
            % Pareto weights y: 1st agent z: 2nd agent
            
	        yy = y(n); zz = z(n);
            
            vsum = yy+zz;
            
            Valbarbar(s,n) = yy*w2(s,1)+zz*w2(s,2);
            
            Valup(s,n)     = (yy+zz)*(-1.5);
        end
        
        
end



% Now compute Valbar by applying the B operator


Val = Valbarbar;

for s=1:ns
   for n=1:Tdir
       for i=1:nv
            q_opt_iter(s,n,i)         = 0;
       end
       for i=1:ns
            for j=1:np
                ropt(s,n,(i-1)*np+j)   = 0;
            end
       end
   end
end


tic

for s = state
    
        is = s;
        
        
        for n = 1:Tdir
            
            % Pareto weights y: 1st agent z: 2nd agent
            yy = y(n); zz = z(n);
            % run snopt
            
            in = n;
            
            
            [q_opt , F_opt, Info] = snoptmain0;

            %dtrack
	         %Info
    
	           
            
            % keep iteration value
            
            Valbar(s,n)               = F_opt(1);
            
            Deps(s,n)                 = Valbar(s,n)-Valbarbar(s,n);
            
            q_opt_iter(s,n,:)         = q_opt;          
                         
            Info_stor(1,s,n)          = Info;
            
           
               
        end
        
end

toc



% Check that Valbar > Valbarbar.    


DB = min(min(Deps))


% Compute upper boound for modulus of contraction.


for s = state
        
        for n = 1:Tdir
                rhomat(s,n) = 1 - (DB/(Valup(s,n)-Valbar(s,n)));
        end
end


% Record and print to screen modulus of contraction.


rho = max(max(rhomat))

% Also record and print to screen factor which is used to calculate error
% bounds.


factor = 1/(1-rho)



% Early runs printed output here for checking purposes.


%q_opt
%F_opt
%Info
%th
            
%Jtrack
%Qtrack
%Vtrack
            
%rope
            
%DG






% % D contains the value function.
% Coordinates:       s  = shock
%                    i  = direction index, ic.

% Initialize value functions for main run.

% If a first run:

if ifirst == 0

Val      = rad*ones(ns,Tdir);
Val      = Valbar;
Val_iter = zeros(size(Val)); % k+1 iteration value function

% If a rerun:

elseif ifirst == 1

load store_data.mat Val Qopt_stor Info_stor

Val

else
    print 'You did not set ifirst'
    stop
    
end


Info_star = [];


% MAIN LOOP!!!!!

% Start timer.

tic

while start == true
    
    iter=iter+1;
    
    
    for s = state
        
        is = s;
        
        for n = 1:Tdir
            
            in = n;
            
            % Pareto weightsyx: 1st agent z: 2nd agent
            yy = y(n); zz = z(n);
            
            
            % run snopt!!!!
            
            
            [q_opt , F_opt, Info] = snoptmain0;
            
            
             
            % Record optimal value.
            
            Val_iter(s,n)               = F_opt(1);
            
            % Calculate terms necessary for calculation of Thompson metric.
            
            top = Val_iter(s,n)-Valbarbar(s,n);
            bot = Val(s,n)     -Valbarbar(s,n);
            
            rhos = log(top/bot);
            
            if rhos < 0
                rhos = -1*rhos;
            end
            
            rhod(s,n) = rhos;
            
            % Record policy.
            
            
            q_opt_iter(s,n,:)           = q_opt;
            
            
            % Record SNOPT information ticket.
            
            Info_stor(iter+1,s,n)       = Info;
                   
            
            % Store all optimal values.
            
            Val_rec(iter,s,n)           = F_opt(1);
            
        end
        
    end
    
    % Compute Thompson metric between current and last value function
    % iterate.
    
    rhomax(iter)  = max(max(rhos))
    
    % Print value function to screen.
    
    Val
    
    % Record Thompson metric value.
    
    rm            = rhomax(iter);
    
    % Calculate usual uniform metric.
    
    Rstor(iter)   = max(max(abs(Val_iter-Val)))
    
    % Test for convergence.
    
    
    if max(max(abs(Val_iter-Val)))<tol || iter >= Maxiter
        start = false;
    end
    
    % Update value function.
   
    Val           = Val_iter;
    
    
    
end




% Switch off timer.

toc

% Main Loop ends!


% View Thompson metrics and error bound estimate.

rhomax

THerrorbD = factor*rm



% Store Data


Qopt_stor = q_opt_iter;

iters = [1:1:iter]


% Save important things to do store_data.mat 

% Note: This must be adjusted manually so that files are sent
% to the correct output directory.

save Output_theta8_alpha15/store_data_th5_a5.mat Val Valbar Qopt_stor Info_stor

save Output_theta8_alpha15/param_th5_a5.mat theta alpha w gamma delta Q ns np ndir

save Output_theta8_alpha15/rhomax_th5_a5 iters rhomax

save Output_theta8_alpha15/Vals_th5_a5.mat Val_rec Valbarbar Valup





%stop

