% -----------------------------------------------------------------
%
% plotgraph_2Ag.m
%
% -----------------------------------------------------------------

% This file plots figures for the two agent case used in 
% Dual Approach to Recursive Optimization: Theory and Examples.
% by Pavoni-Sleet-Messner. It uses data generated by Main_2Agents.m



% ------------------------------------------------------------------
% Initial set up.
% ------------------------------------------------------------------

% Clear and close. Set any global variables. Switch the time on.

clear all;
close all;
clc;
global xx yy zz s Val  delta gamma Q w alpha n ns np Dir cparam th
global ropt is in
tic

% -------------------------------------------------------------------
% load data. 
% -------------------------------------------------------------------



load store_data_th5_a5.mat Val Valbar Qopt_stor Info_stor

load param_th5_a5.mat theta alpha w gamma delta ns np ndir Q

load rhomax_th5_a5 iters rhomax

load Vals_th5_a5.mat Val_rec Valbarbar Valup




th    = (theta-1)/theta;
nv    = np*(1+ns); % Number of variables.


% Create the initial unit quarter-hyper-circle

cen = [0 0 0]; 
rad = 2.2;                      % center radius.
inc = (90/ndir).*[1:1:ndir];    % increment in the angle.

% phi is the inclination
% psi is the azimuth

% Phi determines the split between 1,2 and 3
% A low phi gives less to 1 and 2 and more to 3.

% In our graphs phi will be called phi_2.

% Psi gives split between 1 and 2. 
% A low psi gives more to 1.

% In our graphs psi will be called phi_1.

psi = [0 inc];

%phi and psi are in degrees. To express in radians:


psir = degtorad(psi);

% To normalize to 1.

psi1n = psi./inc(ndir);

psia  = 90 -psi;
psi1na=psia./inc(ndir);

psi1n=psi1na;


Dir = [];  % direction
Cir = [];  % circle


% Set hyperplane directions.


    for j=1:ndir+1
        
        
        
        y(j)   = cosd(psi(j)); 

        z(j)   = sind(psi(j));
        
        phi(j) = y(j)/(y(j)+z(j));

   
end

% Record totals.

ic    = ndir + 1;
Tdir  = ic;


% Collect directions. Dir is 3* 2551 (= g(g+1) + 1)

% Note: Dir(i) * (x' y' z') = rad, is tangent to circle

% with normal Dir(i) and intercepts rad/Dir(i,j).

Dir  = [y; z]




% Transition probability matrix

state = [1:1:ns];     % three states






q_opt_iter = Qopt_stor;


% Record policies in form convenient for graphing/extraction.

for k=1:ns
      for j=1:ndir+1
          ic = j;
            Qthink(k,j,:) = q_opt_iter(k,ic,:);
      end
end





% Next block calculates optimal consumption.

% It also puts policy functions in convenient form for graphing.

% Loop through shocks, directions

for k=1:ns

    
% Directions.    
    
    for j = 1:ndir+1
        
        ic = j;
        
        zt = 0;
  
        
        % Get updated costates. (In quasilinear problem these variables are
        % not optimal choices, but must be computed).
        
        for l = 1:np
        
            % zx(l) = Dir(l,ic)+q_opt_iter(k,ic,l);
            
            zx0(l) = Dir(l,ic)+q_opt_iter(k,ic,l);
            
            for jj=1:ns
            
                zx((l-1)*ns + jj)  = q_opt_iter(k,ic,np+(l-1)*ns+jj);
        
            end
            
            zz(l) = zx0(l)^(1/alpha);
           
            zt    = zt + zz(l);
        end
        
        % Record payoff
        
        valuesv(k,j) = Val(k,ic);
        
        % Record consumption
        
        
        for l=1:np % Player
        
            con(k,j,l)     = gamma(k)*zz(l)/zt;
        
            for jj=1:ns  % Next period shock.
                zweight(k,j,l, jj) = zx((l-1)*ns+jj);
            end          % End next perio shock.
            
        end              % End Player
      
        
    end                  % End directions
    end
    






% End current shock loop




% -------------------------------------------------------
%
%  Policy Graphs follow.
%
% ----------------------------------------------------------

    


%--------------------------------------------------------------------------
%   Set Parameters for figures. (Will make typset consistent with paper).
%--------------------------------------------------------------------------
fName = 'Palatino';
fSize = 12;
f2Size= 9;

%   Paper size and position for single figures
pSize1     = [4.2,3.2];
pPosition1 = [0.1,0.1,4.0,3.0];

%   Paper size and position for double figures
pSize2     = [3.2,3.0];
pPosition2 = [0,0,3.2,3.0];

% Extract consumption for a particular state
% and individual.

   
    for j=1:ndir+1
        
        Val1(j)=  valuesv(1,j);

        con11(j)= con(1,j,1);
        con12(j)= con(1,j,2);
        

        con21(j)= con(2,j,1);
        con22(j)= con(2,j,2);
       
    end
    
 



figure (1)
plot(psi1n,con11',psi1n,con12','--');
xl = xlabel('$\psi$','Interpreter','latex');
yl = ylabel('Consumption','Interpreter','latex');
axis([0 1 0.2 1]);
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/Con1.pdf') ;


% ---------------------------------------------------------------------
% Incentive multiplier policy function. Agent 2, State 2.
% --------------------------------------------------------------------


mult11 = Qthink(1,:,1);
mult12 = Qthink(1,:,2);

figure(2)
plot(psi1n,mult11',psi1n,mult12','--')
xl = xlabel('$\psi$','Interpreter','latex');
yl = ylabel('$m$','Interpreter','latex');
axis([0 1 0 1.5]);
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
%print ('-dpdf', 'FiguresEg/Mult1.pdf') ;



ic111 = Qthink(1,:,3);


% Normalize costates. (It's the normalized costates that affect later
% choices).


k=1;


    for j=1:ndir+1
        for jj=1:ns
           zt = 0;
           for l=1:np
                zt = zt + (zweight(k,j,l, jj)*zweight(k,j,l, jj));
           end
                zt = zt^0.5;
           for l=1:np
                zws(j,l,jj) = zweight(k,j,l,jj)/zt;
           end
        end
    end


ic111 = zws(:,1,1);
ic121 = zws(:,2,1);
ic1   = ic111+ic121;


ic111 = ic111./ic1;
ic121 = ic121./ic1;


figure(3)
plot(phi,ic111',phi,ic121', '--',phi,phi,':')
xl = xlabel('$\psi$','Interpreter','latex');
yl = ylabel('Costate','Interpreter','latex');
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/NCos11.pdf') ;



ic122 = zws(:,2,2);
ic112 = zws(:,1,2);
ic1   = ic122+ic112;
ic122 = ic122./ic1;
ic112 = ic112./ic1;



k=3;


    for j=1:ndir+1
        for jj=1:ns
           zt = 0;
           for l=1:np
                zt = zt + (zweight(k,j,l, jj)*zweight(k,j,l, jj));
           end
                zt = zt^0.5;
           for l=1:np
                zws(j,l,jj) = zweight(k,j,l,jj)/zt;
           end
        end
    end


ic113 = zws(:,1,3);
ic123 = zws(:,2,3);
ic3   = ic113+ic123;
ic113 = ic113./ic3;
ic123 = ic123./ic3;


figure(4)
plot(phi,ic113',phi,ic123','--',phi,phi,':')
xl = xlabel('$\psi$','Interpreter','latex');
yl = ylabel('Costate','Interpreter','latex');
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
%print ('-dpdf', 'FiguresEg/NCos33.pdf') ;



for i=1:ndir+1
    Vr1(i) = Val_rec(1,1,i);
    Vr10(i) = Val_rec(10,1,i);
    Vr25(i) = Val_rec(25,1,i);
    
    %Vr50(i)= Val_rec(50,1,i);
    Valb(i) = Valbar(1,i);
    Valu(i) = Valup(1,i)*1.5;
end



figure(5)
plot(phi, Valu', 'o', phi, Valb', 'o', phi,Vr1',phi,Vr10',':',phi,Vr25','-.')
xl = xlabel('$\psi$','Interpreter','latex');
yl = ylabel('Value Function','Interpreter','latex');
axis([0 1 -4 -2]);
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/Valsb.pdf') ;





% --------------------------------------------------------------
% Now construct simulation.
% --------------------------------------------------------------


% Length of simulation.

nsim = 1000;

% Random number sequence.

rno = rand(nsim,1);

% Generate shocks.

shock(1) = 1;

for i=1:nsim

    if shock(i) == 1
        if rno(i) < 0.9
            shock(i+1) = 1;
        elseif (rno(i) >= 0.9) & (rno(i) <=0.95 )
             shock(i+1) = 2;
        else
              shock(i+1) =3;
        end
    elseif shock(i) == 2
        if rno(i) < 0.9
            shock(i+1) = 2;
        elseif (rno(i) >= 0.9) & (rno(i) <=0.95)
             shock(i+1) = 1;
        else
              shock(i+1) =3;
        end
    else   
        if rno(i) < 0.9
            shock(i+1) = 3;
        elseif (rno(i) >=0.9) & (rno(i) <=0.95)
             shock(i+1) = 1;
        else
              shock(i+1) =2;
        end
    end
    
    
end
    
    

% Choose a start shock and direction.

k=1;
j=4;

ic = j;

% Select corresponding start direction.

wsim(1,:) = Dir(:,ic)';
csim(1)   = con(k,ic,1);

weight    = wsim(1,:);
ws        = sum(weight);
psim(1,:) = weight/ws;


% Recover period 2 weights from policy function and renormalize.

jj             = shock(2);

for l=1:np
    weight(l)  = zweight(k,j,l, jj);
    w2(l)      = weight(l)^2;
end

wNorm          = sum(w2)^0.5;

for l=1:np
    weight(l) = weight(l)/wNorm;
end


wsim(2,:) = weight;
ws        = sum(weight);
psim(2,:) = weight/ws;


    
    
for i=2:nsim-1
    
% Enter with normalized weights. 
% Convert to coordinate (psi,r), r=1.

weight = wsim(i,:);

phis = acos(weight(1));

phis = radtodeg(phis);

   
% Here is shock.

k = shock(i);
jj= shock(i+1);

% Evaluate policy function on this grid.
% Hence generate new directions.

for l=1: np

    
% Load policy function for each player into V.    
    
V = zweight(k,:,l,jj); 
V2= con(k,:,l);
 
V = V';   
V2= V2';

% Test the next line by evaluating at phi(2),psi(3)

%zweight(k,2,3,l)

%w2 = interp2(Phiv,Psiv,V,phi(2),psi(3))

%stop

weight(l) = interp1(psi,V,phis);
cweight(l)= interp1(psi,V2,phis);

end


% Renormalize


for l=1:np
    w2(l)      = weight(l)^2;
end
wNorm          = sum(w2)^0.5;
for l=1:np
    weight(l) = weight(l)/wNorm;
end

% Record

wsim (i+1,:) = weight;
ws           = sum(weight);
psim(i+1,:)  = weight/ws;
csim(i)     = cweight(1);

% Return


    
end    
% ---------------------------------------------------------------------
% Simulation plot.
% --------------------------------------------------------------------

nT = 500;

time = [1:1:nT];





% ---------------------------------------------------------------------
% Thompson metric plot. (Shows convergence).
% --------------------------------------------------------------------

figure(6)
axes('YGrid','on','XGrid','on', 'FontName',fName,'FontSize',fSize)
plot(iters,rhomax)
yl = ylabel('Thompson metric','Interpreter','latex');
xl = xlabel('Iteration','Interpreter','latex');
axis([0 40 0 0.5]);
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
%print ('-dpdf', 'Converge.pdf') ;



figure(7)
plot(time(1:250),shock(1:250))
yl = ylabel({'Shock'; ' '},'Interpreter','latex');
xl = xlabel('Time','Interpreter','latex');
title('Shocks','FontName',fName,'FontSize',fSize)
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'Simul22.pdf') ;



figure(8)
plot(time(1:249), csim(1:249)')
yl = ylabel('$a^1$','Interpreter','latex');
xl = xlabel('Time','Interpreter','latex');
title('Agent 1''s Consumption','FontName',fName,'FontSize',fSize)
axis([0 250 0.5 0.9]);
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'Simul4.pdf') ;





