 function [F, G] = snoptuserfun0(q)

 % Defines the nonlinear part of the function and derivatives


global xx yy zz s Val delta gamma Q w alpha n ns np Dir q_optlin s2
global th Jtrack Vtrack Qtrack rope DG dtrack FCtrack
global ndir

% These variables are sent to snoptmainlin/snoptuserfunlin

global Val2 Dir2 cparam

global is in ropt


% Nonlinear part of the objective and the constraint functions.
% The values of the linear rows are set to 0 (they are not used)

% There are nine choice variables:

% Variables 1 to 2 are the current multipliers of the 2 agents.

% Variables 2 + (i-1)*ns +1 to 2 + i*ns are the next period costates of
% agent i.


% yy = is the Pareto weight of the first agent. 
% zz = is the Pareto weight of the second agent.

% For convenience respecify these:

zstate(1) = yy;
zstate(2) = zz;




% ------------------------------------------
% Evaluate the current dual value function.
% ------------------------------------------

% Note we minimize over (static) resource multiplier and plug in.

% Hence, obtain (slightly modified) version of equation (7) in paper.

% Preliminaries.

% Recall: gamma is endowment size.

eps      = 0.005;

% Define coefficient for current dual function.

coeff   = (1-delta)/(1-alpha);
coeff   = coeff.*gamma(s)^(1-alpha);


% Define aggregator for current dual function.

CES     = @ (y,z) (y.^(1/alpha)+ z.^(1/alpha))^alpha;

for i=1:np 
    z(i)    = zstate(i) + q(i);
end

% Evaluation.

% To avoid explosive derivatives, we approximate current dual at
% very low values of z.

for i=1:np
    ze(i) = max(eps,z(i));
    xe(i) = min(eps,z(i));
end

cs   = CES(ze(1),ze(2));
ap   = (alpha-1)/alpha;

addon = 0;

for i=1:np
    dcs(i) = (cs/ze(i))^ap;
    addon  = addon + (z(i)-ze(i))*dcs(i);
end

J    = coeff*cs + coeff*addon;

% If you want to keep track of J across runs, globalize it here.

Jtrack = J;


    

for i=1:np
	DJ(i) =  coeff*dcs(i);
end



% ---------------------------------------------
% Evaluate continuation value function.
% ----------------------------------------------

% First create (normalized) continuation weights.



for i=1:ns
    for ji=1:np
        valg(i,ji) = delta*q(np+(ji-1)*ns+i)*Q(s,i);
    end
end



% To evaluate the value function we must perform an optimization.

% To speed things up we use a quadratic optimization (i.e. a smooth
% perturbed LP).

Valp  = 0;

DValp = zeros(np,1);

for i=1:ns
    
    % The objective uses normalized updated Pareto weights.

    % We globalize these.
    
    fv2   =  -valg(i,:);
    
    H     =  cparam*eye(np);

    A     =  Dir';
 
    b     =  Val(i,:)';
    
    Aeq   =  [];
    beq   =  [];
    ub    =  zeros(np,1);
    lb    =  [];
    
    for   ji=1:np
        x0(ji) =ropt(is,in,(i-1)*np+ji);
    end
    
    
    options = optimoptions('quadprog', 'Algorithm', 'active-set', 'Display','off');
    
    % Now call quadprog.    
    
    [rop , F_optlin, ExitFlag] = quadprog(H,fv2,A,b,Aeq,beq,lb,ub,x0,options);
    
    for jj=1:np
        ropt(is,in,(i-1)*np+jj) = rop(jj);
    end
    
    d(i)      = -F_optlin;
    
    Valp      = Valp + d(i);
    
    
    
   rope(i,:) = rop; 
    
    
	for ji=1:np
	    DValp(i,ji) = delta*Q(s,i)*rop(ji);
    end
end

dtrack = d;

objective =   J  - q(1)*w(s,1) - q(2)*w(s,2)  + Valp;

Vtrack = Valp;
Qtrack = - q(1)*w(s,1) - q(2)*w(s,2)  ;

F(1)      =   objective;

% Constraints follow. F(2), ..., F(np+1)

% Along with derivatives. DF(i,j).

eps = 0.01;

% Record costates onto qb.

for i=1:ns
 for ij=1:np
    qb(i,ij) = max(q(np+(ij-1)*ns+i),eps);
end
end

% There is a constraint for each player ij.

for ij=1:np
    
    FP0 = 0;
    
    for i =1:ns
        FP(i)         = qb(i,ij)^(1/th);
        FP0           = FP0 + FP(i)*Q(s,i);
    end

    F(ij+1)           = ( FP0 )^th ;

    DF(ij+1,ij  )     = -1;
    
    k                 =  (ij-1)*ns;

    for l = 1:ns
        DF(ij+1,np+k+l)   = (F(ij+1)/FP0)*(FP(l)/qb(l,ij))*Q(s,l);
        F(ij+1)           = F(ij+1) + (q(np+k+l)-qb(l,ij))*DF(ij+1,np+k+l);
    end
        

    F(ij+1)        = F(ij+1) - zstate(ij) - q(ij);
    
 

end


F = F';


% Load objective and constraint derivatives onto G.

% Update this manually as ns and np are changed.



G = [1,  1, DJ(1)-w(s,1) ;
     1,  2, DJ(2)-w(s,2) ;
     1,  3, DValp(1,1)   ;
     1,  4, DValp(2,1)   ;
     1,  5, DValp(3,1)   ;
     1,  6, DValp(1,2)   ;
     1,  7, DValp(2,2)   ;
     1,  8, DValp(3,2)   ;
     2,  1, DF(2,1)      ;
     2,  3, DF(2,3)      ;
     2,  4, DF(2,4)      ;
     2,  5, DF(2,5)      ;
     3,  2, DF(3,2)      ;
     3,  5, DF(3,6)      ;
     3,  6, DF(3,7)      ;
     3,  7, DF(3,8)      ];
 
 DG = G;


% Extract key column. 
 
 
 G = G(:,3);
 
 
 
 


    
