% -----------------------------------------------------------------
%
% plotgraph_3Ag.m
%
% -----------------------------------------------------------------

% This file plots figures for the three agent case used in 
% Dual Approach to Recursive Optimization: Theory and Examples.
% by Pavoni-Sleet-Messner. It uses data generated by Main_3Agents.m



% ------------------------------------------------------------------
% Initial set up.
% ------------------------------------------------------------------

% Clear and close. Set any global variables. Switch the time on.
% Initial set up.

clear all;
close all;
clc;
global xx yy zz s Val  delta gamma Q w alpha n ns np Dir cparam th
global ropt is in
tic

% load data. 

% Note: older versions loaded in different input files (and those files had

% different elements).

% This program will only run properly if correct input file loaded.

% Make sure correct

% -------------------------------------------------

load store_data_th5_a5.mat Val Valbar Qopt_stor Info_stor

load param_th5_a5.mat theta alpha w gamma delta ns np ndir Q

load rhomax_th5_a5 iters rhomax

% -------------------------------------------------


% Note older runs didn't save iters. Enter manually if necessary.

%iters = [1:1:31];

th    = (theta-1)/theta;
nv    = np*(1+ns); % Number of variables.


% Create the initial unit quarter-hyper-circle

cen = [0 0 0]; 
rad = 2.2;                      % center radius.
inc = (90/ndir).*[1:1:ndir];    % increment in the angle.

% phi is the inclination
% psi is the azimuth

% Phi determines the split between 1,2 and 3
% A low phi gives less to 1 and 2 and more to 3.

% In our graphs phi will be called phi_2.

% Psi gives split between 1 and 2. 
% A low psi gives more to 1.

% In our graphs psi will be called phi_1.

phi = [0 inc];
psi = [0 inc];

%phi and psi are in degrees. To express in radians:

phir = degtorad(phi);
psir = degtorad(psi);

% To normalize to 1.

phi1n = phi./inc(ndir);
psi1n = psi./inc(ndir);

psia  = 90 -psi;
psi1na=psia./inc(ndir);


Dir = [];  % direction
Cir = [];  % circle


% Set hyperplane directions.

for i = 2:ndir+1
    for j=1:ndir+1
        
        ic = (i-2)*(ndir+1)+j;
        
        y(ic)   = sind(phi(i))*cosd(psi(j)); 

        z(ic)   = sind(phi(i))*sind(psi(j));

        x(ic)   = cosd(phi(i));
        
        
    end
end

% Record totals.

ic    = ndir*(ndir+1) + 1;
Tdir  = ic;

% Add the (0,0,1) direction. (Remember that there is a redundancy in
% the mapping to hyperplane directions. T avoid this, we add this
% final direction manually rather than generate it multiple times
% off of a grid.


x(ic) = 1;
y(ic) = 0;
z(ic) = 0;


% Collect directions. Dir is 3* 2551 (= g(g+1) + 1)

% Note: Dir(i) * (x' y' z') = rad, is tangent to circle

% with normal Dir(i) and intercepts rad/Dir(i,j).

Dir  = [x; y; z]




% Transition probability matrix

state = [1:1:ns];     % three states






q_opt_iter = Qopt_stor;


% Record policies in form convenient for graphing/extraction.

for k=1:ns
  for i =2:ndir+1
      for j=1:ndir+1
          ic = (i-2)*(ndir+1) +j;
            Qthink(k,i,j,:) = q_opt_iter(k,ic,:);
      end
  end
  i =1
  for j=1:ndir+1
          ic = Tdir;
            Qthink(k,i,j,:) = q_opt_iter(k,ic,:);
      end
end




% Next block calculates optimal consumption.

% It also puts policy functions in convenient form for graphing.

% Loop through shocks, directions

for k=1:ns

    
% Directions.    
    
    
    for i = 2:ndir+1
    for j = 1:ndir+1
        
        ic = (i-2)*(ndir+1)+j;
        
        zt = 0;
  
        
        % Get updated costates. (In quasilinear problem these variables are
        % not optimal choices, but must be computed).
        
        for l = 1:np
        
            % zx(l) = Dir(l,ic)+q_opt_iter(k,ic,l);
            
            zx0(l) = Dir(l,ic)+q_opt_iter(k,ic,l);
            
            for jj=1:ns
            
                zx((l-1)*ns + jj)  = q_opt_iter(k,ic,3+(l-1)*ns+jj);
        
            end
            
            zz(l) = zx0(l)^(1/alpha);
           
            zt    = zt + zz(l);
        end
        
        % Record payoff
        
        valuesv(k,i,j) = Val(k,ic);
        
        % Record consumption
        
        
        for l=1:np % Player
        
            con(k,i,j,l)     = gamma(k)*zz(l)/zt;
        
            for jj=1:ns  % Next period shock.
                zweight(k,i,j,l, jj) = zx((l-1)*ns+jj);
            end          % End next perio shock.
            
        end              % End Player
      
        
    end                  % End directions
    end
    





    i=1;

    for j=1:ndir+1
    
        ic = ndir*(ndir+1) + 1;
    
        % The weights are:
        
            zt = 0;
        
            for l = 1:np
        
                zx0(l) = Dir(l,ic)+q_opt_iter(k,ic,l);
        
                zz(l) = zx0(l)^(1/alpha);
           
                zt    = zt + zz(l);
                
                for jj=1:ns
            
                    zx((l-1)*ns + jj)  = q_opt_iter(k,ic,3+(l-1)*ns+jj);
        
                end
            end    
            
            for l=1:np
        
                con(k,i,j,l)     = gamma(k)*zz(l)/zt;
                
                for jj=1:ns  % Next period shock.
                    zweight(k,i,j,l, jj) = zx((l-1)*ns+jj);
                end  
        
            end
        
            valuesv(k,i,j) = Val(k,ic);
        
    end

% End current shock loop

end

k=1;

for i = 2:ndir+1
    for j = 1:ndir+1
        
        ic = (i-2)*(ndir+1)+j;
        
        % for each future state
        
        for jj=1:ns
        
        zt(k,i,j,jj) =0;
        
        for l=1:np  % Next period shock.
                    zt(k,i,j,jj) = zt(k,i,j,jj) ...
                        + (q_opt_iter(k,ic,3+(l-1)*ns+jj)^2);
        end  

        zt(k,i,j,jj) = zt(k,i,j,jj)^(1/2);

        for l=1:np
            zwn(k,i,j,jj,l) = q_opt_iter(k,ic,3+(l-1)*ns+jj)/zt(k,i,j,jj);
        end
        
        px(k,i,j,jj) = acosd(zwn(k,i,j,jj,1));
        py(k,i,j,jj) = acosd(zwn(k,i,j,jj,2)/sind(px(k,i,j,jj)));
        
        dpx(k,i,j,jj)= px(k,i,j,jj)-phi(i);
        dpy(k,i,j,jj)= py(k,i,j,jj)-psi(j);
        
        end
    end
end

i=1;

    for j = 1:ndir+1
        
        ic = ndir*(ndir+1) + 1;
        
        % for each future state
        
        for jj=1:ns
        
        zt(k,i,j,jj) =0;
        
        for l=1:np  % Next period shock.
                    zt(k,i,j,jj) = zt(k,i,j,jj) ...
                        + (q_opt_iter(k,ic,3+(l-1)*ns+jj)^2);
        end  

        zt(k,i,j,jj) = zt(k,i,j,jj)^(1/2);

        for l=1:np
            zwn(k,i,j,jj,l) = q_opt_iter(k,ic,3+(l-1)*ns+jj)/zt(k,i,j,jj);
        end
        
        px(k,i,j,jj) = acosd(zwn(k,i,j,jj,1));
        py(k,i,j,jj) = acosd(zwn(k,i,j,jj,2)/sind(px(k,i,j,jj)));
        
        dpx(k,i,j,jj)= px(k,i,j,jj)-phi(i);
        dpy(k,i,j,jj)= py(k,i,j,jj)-psi(j);
        
        end
    end






for i=1:ndir+1
    for j=1:ndir+1
        dpx1(i,j)=dpx(1,i,j,1);
        dpy1(i,j)=dpy(1,i,j,1);
    end
end

phi0=phi/90;
psi0=psi/90;

[phi2,psi2] = meshgrid(phi0,psi0);



%--------------------------------------------------------------------------
%   Set Parameters for figures. (Will make typset consistent with paper).
%--------------------------------------------------------------------------
fName = 'Palatino';
fSize = 12;
f2Size= 9;

%   Paper size and position for single figures
pSize1     = [4.2,3.2];
pPosition1 = [0.1,0.1,4.0,3.0];

%   Paper size and position for double figures
pSize2     = [3.2,3.0];
pPosition2 = [0,0,3.2,3.0];




figure(1)

quiver(phi2,psi2,dpx1',dpy1')
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
%print ('-dpdf', 'FiguresEg/quiver1.pdf') ;



for i=1:ndir+1
    for j=1:ndir+1
        dpx12(i,j)=dpx(1,i,j,2);
        dpy12(i,j)=dpy(1,i,j,2);
    end
end





k=2;

for i = 2:ndir+1
    for j = 1:ndir+1
        
        ic = (i-2)*(ndir+1)+j;
        
        % for each future state
        
        for jj=1:ns
        
        zt(k,i,j,jj) =0;
        
        for l=1:np  % Next period shock.
                    zt(k,i,j,jj) = zt(k,i,j,jj) ...
                        + (q_opt_iter(k,ic,3+(l-1)*ns+jj)^2);
        end  

        zt(k,i,j,jj) = zt(k,i,j,jj)^(1/2);

        for l=1:np
            zwn(k,i,j,jj,l) = q_opt_iter(k,ic,3+(l-1)*ns+jj)/zt(k,i,j,jj);
        end
        
        px(k,i,j,jj) = acosd(zwn(k,i,j,jj,1));
        py(k,i,j,jj) = acosd(zwn(k,i,j,jj,2)/sind(px(k,i,j,jj)));
        
        dpx(k,i,j,jj)= px(k,i,j,jj)-phi(i);
        dpy(k,i,j,jj)= py(k,i,j,jj)-psi(j);
        
        end
    end
end

i=1;

    for j = 1:ndir+1
        
        ic = ndir*(ndir+1) + 1;
        
        % for each future state
        
        for jj=1:ns
        
        zt(k,i,j,jj) =0;
        
        for l=1:np  % Next period shock.
                    zt(k,i,j,jj) = zt(k,i,j,jj) ...
                        + (q_opt_iter(k,ic,3+(l-1)*ns+jj)^2);
        end  

        zt(k,i,j,jj) = zt(k,i,j,jj)^(1/2);

        for l=1:np
            zwn(k,i,j,jj,l) = q_opt_iter(k,ic,3+(l-1)*ns+jj)/zt(k,i,j,jj);
        end
        
        px(k,i,j,jj) = acosd(zwn(k,i,j,jj,1));
        py(k,i,j,jj) = acosd(zwn(k,i,j,jj,2)/sind(px(k,i,j,jj)));
        
        dpx(k,i,j,jj)= px(k,i,j,jj)-phi(i);
        dpy(k,i,j,jj)= py(k,i,j,jj)-psi(j);
        
        end
    end






for i=1:ndir+1
    for j=1:ndir+1
        dpx22(i,j)=dpx(k,i,j,2);
        dpy22(i,j)=dpy(k,i,j,2);
    end
end







figure(2)

quiver(phi2,psi2,dpx22',dpy22')
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
%print ('-dpdf', 'FiguresEg/quiver22.pdf') ;



for i=1:ndir+1
    for j=1:ndir+1
        dpx21(i,j)=dpx(k,i,j,1);
        dpy21(i,j)=dpy(k,i,j,1);
    end
end

[phi2,psi2] = meshgrid(phi,psi);

  
        
        


% -----------------------------------------------------------
% First plots of consumption policy function.
% -----------------------------------------------------------

% Extract consumption for a particular state
% and individual.

for i=1:ndir+1
   
    for j=1:ndir+1
        
        Val1(i,j)=  valuesv(3,i,j);

        con11(i,j)= con(1,i,j,1);
        con12(i,j)= con(1,i,j,2);
        con13(i,j)= con(1,i,j,3);

        con21(i,j)= con(2,i,j,1);
        con22(i,j)= con(2,i,j,2);
        con23(i,j)= con(2,i,j,3);

        con31(i,j)= con(3,i,j,1);
        con32(i,j)= con(3,i,j,2);
        con33(i,j)= con(3,i,j,3);
    end
    
end

phi2 = phi(1:ndir+1);

[p2, q2] = meshgrid(phi2,psi); 


phi2n = phi1n(1:ndir+1);

[p2n, q2n] = meshgrid(phi2n,psi1n); 



% --------------------------------------------------------------
% Now construct simulation.
% --------------------------------------------------------------


% Length of simulation.

nsim = 1000;

% Random number sequence.

rno = rand(nsim,1);

% Generate shocks.

shock(1) = 1;

for i=1:nsim

    if shock(i) == 1
        if rno(i) < 0.8
            shock(i+1) = 1;
        elseif (rno(i) >= 0.8) & (rno(i) <0.9 )
             shock(i+1) = 2;
        else
              shock(i+1) =3;
        end
    elseif shock(i) == 2
        if rno < 0.8
            shock(i+1) = 2;
        elseif (rno(i) >= 0.8) & (rno(i) <0.9)
             shock(i+1) = 1;
        else
              shock(i+1) =3;
        end
    else   
        if rno(i) < 0.8
            shock(i+1) = 3;
        elseif (rno(i) >=0.8) & (rno(i) <0.9)
             shock(i+1) = 1;
        else
              shock(i+1) =2;
        end
    end
    
    
end
    
    

% Choose a start shock and direction coordinates.

k=1;
i=4;
j=4;

ic = (i-2)*(ndir+1)+j;

% Select corresponding start direction.

wsim(1,:) = Dir(:,ic)';
csim(1)   = con(k,i,j,1);

weight    = wsim(1,:);
ws        = sum(weight);
psim(1,:) = weight/ws;


% Recover period 2 weights from policy function and renormalize.

jj        = shock(2);

for l=1:np
    weight(l)  = zweight(k,i,j,l, jj);
    w2(l)      = weight(l)^2;
end

wNorm          = sum(w2)^0.5;

for l=1:np
    weight(l) = weight(l)/wNorm;
end


wsim(2,:) = weight;
ws        = sum(weight);
psim(2,:) = weight/ws;


% Create a meshgrid for the simulation


[Phiv,Psiv] = meshgrid(phi,psi);


    
    
for i=2:nsim-1
    
% Enter with normalized weights. 
% Convert to spherical coordinates (phi,psi,r), r=1.

weight = wsim(i,:);

phis = acos(weight(3));

phis = radtodeg(phis);

if weight(1) > 0
    psis = atan(weight(2)/weight(1));
    psis = radtodeg(psis);
    
else
    psis = 90;
end    
   
% Here is shock.

k = shock(i);
jj= shock(i+1);

% Evaluate policy function on this grid.
% Hence generate new directions.

for l=1: np

    
% Load policy function for each player into V.    
    
V(:,:) = zweight(k,:,:,l,jj);    
 
V = V';  

V2(:,:)= con(k,:,:,l);
  
V2= V2';

% Test the next line by evaluating at phi(2),psi(3)

%zweight(k,2,3,l)

%w2 = interp2(Phiv,Psiv,V,phi(2),psi(3))

%stop

weight(l) = interp2(Phiv,Psiv,V,phis,psis);

cweight(l) = interp2(Phiv,Psiv,V2,phis,psis);


end


% Renormalize


for l=1:np
    w2(l)      = weight(l)^2;
end
wNorm          = sum(w2)^0.5;
for l=1:np
    weight(l) = weight(l)/wNorm;
end

% Record

wsim (i+1,:) = weight;
ws           = sum(weight);
psim(i+1,:)  = weight/ws;
csim(i)      = cweight(1);

% Return


    
end    



% -------------------------------------------------------
%
%  Main Graphs follow.
%
% ----------------------------------------------------------

    


%--------------------------------------------------------------------------
%   Set Parameters for figures. (Will make typset consistent with paper).
%--------------------------------------------------------------------------
fName = 'Palatino';
fSize = 12;
f2Size= 9;

%   Paper size and position for single figures
pSize1     = [4.2,3.2];
pPosition1 = [0.1,0.1,4.0,3.0];

%   Paper size and position for double figures
pSize2     = [3.2,3.0];
pPosition2 = [0,0,3.2,3.0];


% ---------------------------------------------------------------------
% Consumption policy functions.
% ----------------------------------------------------------------------


% Consumption.

% Agent 1, State 2.


figure(3)
axes('YGrid','on','XGrid','on','Zgrid', 'on', 'FontName',fName,'FontSize',fSize)
box on
surf(p2n,q2n,con21')
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
zl = zlabel('$c(2,1)$','Interpreter','latex');
axis([0 1 0 1 0 1.5])
view(128,32);
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(zl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
%print ('-dpdf', 'FiguresEg/con21r2.pdf') ;


% Agent 2, State 2.


figure(4)
axes('YGrid','on','XGrid','on','Zgrid', 'on', 'FontName',fName,'FontSize',fSize)
box on
surf(p2n,q2n,con22')
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
zl = zlabel('$c(2,2)$','Interpreter','latex');
axis([0 1 0 1 0 1.5])
view(-118,34);
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(zl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/con22r2.pdf') ;


% Agent 3, State 2.

figure(5)
axes('YGrid','on','XGrid','on','FontName',fName,'FontSize',f2Size)
box on
surf(p2n,q2n,con23')
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
zl = zlabel('$c(2,3)$','Interpreter','latex');
axis([0 1 0 1 0 1.5])
view(-48,30);
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(zl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/con23r2.pdf') ;




% ---------------------------------------------------------------------
% Simulation plot.
% --------------------------------------------------------------------

nT = 250;

time = [1:1:nT];


figure(6)
plot(time(1:250),shock(1:250))
yl = ylabel({'Shock'; ' '},'Interpreter','latex');
xl = xlabel('Time','Interpreter','latex');
title('Shocks','FontName',fName,'FontSize',fSize)
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/Simul22.pdf') ;

% figure(16)
% %axes(ha(2));
% plot(time(1:250), psim(1:250,1))
% yl = ylabel('Costate','Interpreter','latex');
% xl = xlabel('Time','Interpreter','latex');
% title('Agent 1''s Costate','FontName',fName,'FontSize',fSize)
% set(xl,'FontName',fName,'FontSize',f2Size);
% set(yl,'FontName',fName,'FontSize',f2Size);
% set(gca,'FontName',fName,'FontSize',fSize);
% set(gcf,'paperunits','inches')
% set(gcf,'papersize',pSize1)
% set(gcf,'PaperPosition',pPosition1)
% % print ('-dpdf', 'FiguresEg/Simul23.pdf') ;

figure(7)
%axes(ha(2));
plot(time(1:249), csim(1:249)')
yl = ylabel('$a^1$','Interpreter','latex');
xl = xlabel('Time','Interpreter','latex');
title('Agent 1''s Consumption','FontName',fName,'FontSize',fSize)
axis([0 250 0.2 1])
set(xl,'FontName',fName,'FontSize',f2Size);
set(yl,'FontName',fName,'FontSize',f2Size);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/Simul4.pdf') ;



% ---------------------------------------------------------------------
% Thompson metric plot. (Shows convergence).
% --------------------------------------------------------------------

figure(8)
axes('YGrid','on','XGrid','on', 'FontName',fName,'FontSize',fSize)
plot(iters,rhomax)
yl = ylabel('Thompson metric','Interpreter','latex');
xl = xlabel('Iteration','Interpreter','latex');
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/Converge.pdf') ;


% ---------------------------------------------------------------------
% Dual value function. State 2.
% --------------------------------------------------------------------



for i=1:ndir+1
    for j=1:ndir+1
        valuestar(i,j) = valuesv(2,i,j);
    end
end


figure(9)
axes('YGrid','on','XGrid','on','Zgrid', 'on', 'FontName',fName,'FontSize',fSize)
box on
surf(p2n,q2n,valuestar')
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
zl = zlabel('Values','Interpreter','latex');
view(-140,38);
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(zl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/Values1.pdf') ;



% ---------------------------------------------------------------------
% Incentive multiplier policy function. Agent 2, State 2.
% --------------------------------------------------------------------


VThink(:,:) = Qthink(2,:,:,2);

figure(10)
axes('YGrid','on','XGrid','on','Zgrid', 'on', 'FontName',fName,'FontSize',fSize)
box on
surf(phi1n,psi1n,VThink');
xl = xlabel('$\phi_1$','Interpreter','latex');
yl = ylabel('$\phi_2$','Interpreter','latex');
zl = zlabel('$m^2(2)$ \\ ','Interpreter','latex');
view(45,38);
set(xl,'FontName',fName,'FontSize',fSize);
set(yl,'FontName',fName,'FontSize',fSize);
set(zl,'FontName',fName,'FontSize',fSize);
set(gca,'FontName',fName,'FontSize',fSize);
set(gcf,'paperunits','inches')
set(gcf,'papersize',pSize1)
set(gcf,'PaperPosition',pPosition1)
% print ('-dpdf', 'FiguresEg/eta31n.pdf') ;

