 function [F, G] = snoptuserfun00(q)

 % Defines the nonlinear part of the function and derivatives


global xx yy zz s Val delta gamma Q w alpha n ns np Dir q_optlin s2
global th Jtrack Vtrack Qtrack rope DG co
global ndir

% These variables are sent to snoptmainlin/snoptuserfunlin

global Val2 Dir2 cparam

global is in ropt


% Nonlinear part of the objective and the constraint functions.
% The values of the linear rows are set to 0 (they are not used)

% There are nine choice variables:

% Variables 1 to 3 are the current multipliers of the 3 agents.

% Variables 3 + (i-1)*ns +1 to 3 + i*ns are the next period costates of
% agent i.


% xx = is the Pareto weight of the first  agent. 
% yy = is the Pareto weight of the second agent. 
% zz = is the Pareto weight of the third agent.

% For convenience respecify these:

zstate(1) = xx;
zstate(2) = yy;
zstate(3) = zz;



% ------------------------------------------
% Evaluate the current dual value function.
% ------------------------------------------

% Note we minimize over (static) resource multiplier and plug in.

% Hence, obtain (slightly modified) version of equation (7) in paper.

% Preliminaries.

% Recall: gamma is endowment size.

eps      = 0.005;

% Define coefficient for current dual function.

coeff   = (1-delta)/(1-alpha);
coeff   = coeff.*gamma(s)^(1-alpha);


% Define aggregator for current dual function.

CES     = @ (x,y,z) (x.^(1/alpha)+y.^(1/alpha)+ z.^(1/alpha))^alpha;

for i=1:np 
    z(i)    = zstate(i) + q(i);
end

% Evaluation.

% To avoid explosive derivatives, we approximate current dual at
% very low values of z.

for i=1:np
    ze(i) = max(eps,z(i));
    xe(i) = min(eps,z(i));
end

cs   = CES(ze(1),ze(2),ze(3));
ap   = (alpha-1)/alpha;

addon = 0;

for i=1:np
    dcs(i) = (cs/ze(i))^ap;
    addon  = addon + (z(i)-ze(i))*dcs(i);
    co(i)  = gamma(s)*((ze(i))^(1/alpha))/(cs^(1/alpha));
end

J    = coeff*cs + coeff*addon;



% If you want to keep track of J across runs, globalize it here.

Jtrack = J;


    

for i=1:np
	DJ(i) =  coeff*dcs(i);
end



% ---------------------------------------------
% Evaluate continuation value function.
% ----------------------------------------------

% First create (normalized) continuation weights.



for i=1:ns
    for ji=1:np
        valg(i,ji) = delta*q(3+(ji-1)*ns+i)*Q(s,i);
    end
end



% To evaluate the value function we must perform an optimization.

% To speed things up we use a quadratic optimization (i.e. a smooth
% perturbed LP).

Valp  = 0;

DValp = zeros(3,1);

for i=1:ns
    
    % The objective uses normalized updated Pareto weights.

    % We globalize these.
    
    fv2   =  -valg(i,:);
    
    H     =  cparam*eye(3);

    A     =  Dir';
 
    b     =  Val(i,:)';
    
    Aeq   =  [];
    beq   =  [];
    ub    =  zeros(3,1);
    lb    =  [];
    
    for   ji=1:np
        x0(ji) =ropt(is,in,(i-1)*np+ji);
    end
    
    
    options = optimoptions('quadprog', 'Algorithm', 'active-set', 'Display','off');
    
    % Now call quadprog.    
    
    [rop , F_optlin, ExitFlag] = quadprog(H,fv2,A,b,Aeq,beq,lb,ub,x0,options);
    
    for jj=1:np
        ropt(is,in,(i-1)*np+jj) = rop(jj);
    end
    
    d(i)      = -F_optlin;
    
    Valp      = Valp + d(i);
    
    
    
   rope(i,:) = rop; 
   
    
	for ji=1:np
	    DValp(i,ji) = delta*Q(s,i)*rop(ji);
    end
end



objective =   J  - q(1)*w(s,1) - q(2)*w(s,2) - q(3)*w(s,3) + Valp;

Vtrack = Valp;
Qtrack = - q(1)*w(s,1) - q(2)*w(s,2) - q(3)*w(s,3) ;

F(1)      =   objective;

% Constraints follow. F(2), F(3), F(4).

% Along with derivatives. DF(i,j).

eps = 0.01;

for i=1:np*ns
    qb(i) = max(q(3+i),eps);
end


for i=1:np
    
    k   = (i-1)*ns;  
    
    FP0 = 0;
    
    for l =1:ns
        FP(l)         = qb(k+l)^(1/th);
        FP0           = FP0 + FP(l)*Q(s,l);
    end

    F(i+1)      = ( FP0 )^th ;

    DF(i+1,i  )   = -1;
    
    for l = 1:ns
        DF(i+1,3+k+l)   = (F(i+1)/FP0)*(FP(l)/qb(k+l))*Q(s,l);
        F(i+1)          = F(i+1) + (q(3+k+l)-qb(k+l))*DF(i+1,3+k+l);
    end
        
%    DF(i+1,k+5)   = (F(i+1)/FP)*(FP2/qb(k+2))*Q(s,2);

    F(i+1)      = F(i+1) - zstate(i) - q(i);
    
 

end


F = F';


% Load objective and constraint derivatives onto G.

% Update this manually as ns and np are changed.



G = [1,  1, DJ(1)-w(s,1) ;
     1,  2, DJ(2)-w(s,2) ;
     1,  3, DJ(3)-w(s,3) ; 
     1,  4, DValp(1,1)   ;
     1,  5, DValp(2,1)   ;
     1,  6, DValp(3,1)   ;
     1,  7, DValp(1,2)   ;
     1,  8, DValp(2,2)   ;
     1,  9, DValp(3,2)   ;
     1, 10, DValp(1,3)   ;
     1, 11, DValp(2,3)   ;
     1, 12, DValp(3,3)   ;
     2,  1, DF(2,1)      ;
     2,  4, DF(2,4)      ;
     2,  5, DF(2,5)      ;
     2,  6, DF(2,6)      ;
     3,  2, DF(3,2)      ;
     3,  7, DF(3,7)      ;
     3,  8, DF(3,8)      ;
     3,  9, DF(3,9)      ;
     4,  3, DF(4,3)      ;
     4, 10, DF(4,10)     ;
     4, 11, DF(4,11)     ;
     4, 12, DF(4,12)     ];
 
 DG = G;


% Extract key column. 
 
 
 G = G(:,3);
 
 
 
 


    
